package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.balance.GcbSalaryAccQueryRequest;
import com.huifu.jh.salary.model.balance.GcbSalaryAccQueryResponse;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * 余额查询
 */
@Controller
public class GcbSalaryAccQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造余额查询信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryAccQuery.do")
	public String showGcbSalaryAccQuery(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryAccQuery();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryAccQuery";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryAccQuery() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，核心企业机构号
		map.put("coreMemberId", "148079084486732398");

		return map;
	}
	

	/**
	 * 步骤二、构建余额查询信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryAccQuery.do")
	public String doGcbSalaryAccQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 余额查询服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/queryAccount.htm";
		// 余额查询服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/queryAccount.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryAccQueryRequest queryRequest = new GcbSalaryAccQueryRequest();

		// 必填，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，核心企业机构号
		queryRequest.setCoreMemberId(request.getParameter("coreMemberId"));
		
		// 发送请求
		GcbSalaryAccQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbSalaryAccQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "GcbSalaryAccQueryResponse";
	}

}
