package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.allot.GcbSalaryAllotApplyRequest;
import com.huifu.jh.salary.model.allot.GcbSalaryAllotApplyResponse;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 资金调拨申请
 */
@Controller
public class GcbSalaryAllotApplyController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造资金调拨申请信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryAllotApply.do")
	public String showGcbSalaryAllotApply(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryAllotApply();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryAllotApply";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryAllotApply() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，必须唯一：建议14位以上时间戳+随机数
		map.put("orderId", new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));

		// 条件必填，汇付跨境系统用户号，transType=2必填
		map.put("userAcctId", "");

		// 非必填，1、从商户账户划拨；2、从用户账户划拨；不填默认为1。
		map.put("transType", "");

		// 必填，核心企业机构号
		map.put("coreMemberId", "148079084486732398");

		// 必填，单位为分，无小数位
		map.put("amount", "100");

		return map;
	}
	

	/**
	 * 步骤二、构建资金调拨申请信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryAllotApply.do")
	public String doGcbSalaryAllotApply(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 资金调拨申请服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/allotApply.htm";
		// 资金调拨申请服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/allotApply.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryAllotApplyRequest applyRequest = new GcbSalaryAllotApplyRequest();

		// 必填，必须唯一：建议14位以上时间戳+随机数
		applyRequest.setOrderId(request.getParameter("orderId"));

		// 必填，会员账号，由我司提供的商户号+01，共13位数字
		applyRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 条件必填，汇付跨境系统用户号，transType=2必填
		applyRequest.setUserAcctId(request.getParameter("userAcctId"));

		// 非必填，1、从商户账户划拨；2、从用户账户划拨；不填默认为1。
		applyRequest.setTransType(request.getParameter("transType"));

		// 必填，核心企业机构号
		applyRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 必填，单位为分，无小数位
		applyRequest.setAmount(request.getParameter("amount"));
		
		// 发送请求
		GcbSalaryAllotApplyResponse applyResponse = gcbPayClient.execute(applyRequest);
		logger.info("GcbSalaryAllotApplyResponse => " + new String(applyResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", applyResponse.getParamMap());
		return "GcbSalaryAllotApplyResponse";
	}

}
