package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.allot.GcbSalaryAllotQueryRequest;
import com.huifu.jh.salary.model.allot.GcbSalaryAllotQueryResponse;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * 资金调拨结果查询
 */
@Controller
public class GcbSalaryAllotQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造资金调拨结果查询信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryAllotQuery.do")
	public String showGcbSalaryAllotQuery(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryAllotQuery();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryAllotQuery";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryAllotQuery() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，原请求订单号
		map.put("orderId", "20211028103026629");

		// 必填，核心企业机构号
		map.put("coreMemberId", "148079084486732398");

		return map;
	}
	

	/**
	 * 步骤二、构建资金调拨结果查询信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryAllotQuery.do")
	public String doGcbSalaryAllotQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 资金调拨结果查询服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/allotQuery.htm";
		// 资金调拨结果查询服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/allotQuery.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryAllotQueryRequest queryRequest = new GcbSalaryAllotQueryRequest();

		// 必填，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，核心企业机构号
		queryRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 必填，原请求订单号
		queryRequest.setOrderId(request.getParameter("orderId"));
		
		// 发送请求
		GcbSalaryAllotQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbSalaryAllotQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "GcbSalaryAllotQueryResponse";
	}

}
