package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.contract.GcbSalaryContractQueryRequest;
import com.huifu.jh.salary.model.contract.GcbSalaryContractQueryResponse;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 签约查询
 */
@Controller
public class GcbSalaryContractQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造签约查询信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryContractQuery.do")
	public String showGcbSalaryContractQuery(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryContractQuery();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryContractQuery";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryContractQuery() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，订单号，-
		map.put("orderId", new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));

		// 必填，核心企业会员号，核心企业机构号
		map.put("coreMemberId", "148079084486732398");

		// 条件必填，员工会员号，合同类型为6时必填
		map.put("personMemberId", "");

		// 必填，合同类型，必填   1：核心企业-税优公司 6：个人-税优公司
		map.put("contractType", "1");

		// 必填，开始时间，yyyyMMdd
		map.put("startDate", "20210101");

		// 必填，结束时间，yyyyMMdd
		map.put("endDate", "20211231");

		return map;
	}
	

	/**
	 * 步骤二、构建签约查询信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryContractQuery.do")
	public String doGcbSalaryContractQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 签约查询服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/contractQuery.htm";
		// 签约查询服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/contractQuery.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryContractQueryRequest queryRequest = new GcbSalaryContractQueryRequest();

		// 必填，商户号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，订单号，-
		queryRequest.setOrderId(request.getParameter("orderId"));

		// 必填，核心企业会员号，核心企业机构号
		queryRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 条件必填，员工会员号，合同类型为6时必填
		queryRequest.setPersonMemberId(request.getParameter("personMemberId"));

		// 必填，合同类型，必填   1：核心企业-税优公司 6：个人-税优公司
		queryRequest.setContractType(request.getParameter("contractType"));

		// 必填，开始时间，yyyyMMdd
		queryRequest.setStartDate(request.getParameter("startDate"));

		// 必填，结束时间，yyyyMMdd
		queryRequest.setEndDate(request.getParameter("endDate"));
		
		// 发送请求
		GcbSalaryContractQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbSalaryContractQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "GcbSalaryContractQueryResponse";
	}

}
