package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.credit.GcbSalaryFeeQueryRequest;
import com.huifu.jh.salary.model.credit.GcbSalaryFeeQueryResponse;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 代发费用查询
 */
@Controller
public class GcbSalaryFeeQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造代发费用查询信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryFeeQuery.do")
	public String showGcbSalaryFeeQuery(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryFeeQuery();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryFeeQuery";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryFeeQuery() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，员工会员号，-
		map.put("personMemberId", "555077972985925781");

		// 必填，落地公司会员号，-
		map.put("groundMemberId", "980149419163571612");

		// 必填，订单号，必须唯一：建议14位以上时间戳+随机数
		map.put("orderId", new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));

		// 必填，核心企业会员号，核心企业机构号
		map.put("coreMemberId", "148079084486732398");

		// 必填，代发模式，1-普票 ；2-专票
		map.put("salaryModle", "1");

		// 必填，发放的工资金额，单位为分，无小数位
		map.put("salary", "60000");

		return map;
	}
	

	/**
	 * 步骤二、构建代发费用查询信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryFeeQuery.do")
	public String doGcbSalaryFeeQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 代发费用查询服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/salaryCharge.htm";
		// 代发费用查询服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/salaryCharge.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryFeeQueryRequest queryRequest = new GcbSalaryFeeQueryRequest();

		// 必填，商户号，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，员工会员号，-
		queryRequest.setPersonMemberId(request.getParameter("personMemberId"));

		// 必填，落地公司会员号，-
		queryRequest.setGroundMemberId(request.getParameter("groundMemberId"));

		// 必填，订单号，必须唯一：建议14位以上时间戳+随机数
		queryRequest.setOrderId(request.getParameter("orderId"));

		// 必填，核心企业会员号，核心企业机构号
		queryRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 必填，代发模式，1-普票 ；2-专票
		queryRequest.setSalaryModle(request.getParameter("salaryModle"));

		// 必填，发放的工资金额，单位为分，无小数位
		queryRequest.setSalary(request.getParameter("salary"));
		
		// 发送请求
		GcbSalaryFeeQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbSalaryFeeQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "GcbSalaryFeeQueryResponse";
	}

}
