package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.invoice.GcbSalaryInvoiceApplyRequest;
import com.huifu.jh.salary.model.invoice.GcbSalaryInvoiceApplyResponse;
import com.huifu.jh.salary.model.invoice.GcbSalaryInvoiceResult;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 开票申请
 */
@Controller
public class GcbSalaryInvoiceApplyController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造开票申请信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryInvoiceApply.do")
	public String showGcbSalaryInvoiceApply(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryInvoiceApply();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryInvoiceApply";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryInvoiceApply() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，订单号，必须唯一：建议14位以上时间戳+随机数
		map.put("orderId", new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));

		// 必填，核心企业会员号，
		map.put("coreMemberId", "148079084486732398");

		// 必填，批次列表，代发批次号，多个英文逗号分隔，如1,2,3,4(多个表示合并开一张票)
		map.put("batchList", "144168,897255");

		// 必填，异步地址，异步通知地址
		map.put("asynUrl", "http://localhost:8080/notifyReceiverBgInvoice.do");

		// 必填，接收人手机号，
		map.put("receiveMobile", "13218072315");

		// 必填，接收人姓名，
		map.put("receiveName", "冯浩波");

		// 必填，快递地址，
		map.put("courierAddress", "南京市江宁区");

		// 非必填，购方税号，
		map.put("invoiceTaxNo", "");

		// 非必填，购方公司名称，
		map.put("invoiceName", "");

		// 非必填，购方公司地址，
		map.put("invoiceAddress", "");

		// 非必填，购方公司账号，
		map.put("invoiceCardNum", "");

		// 非必填，购方银行名称，
		map.put("invoiceBankName", "");

		// 非必填，购方联系电话，
		map.put("invoicePhone", "");

		// 非必填，备注，
		map.put("remarks", "");

		return map;
	}
	

	/**
	 * 步骤二、构建开票申请信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryInvoiceApply.do")
	public String doGcbSalaryInvoiceApply(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 开票申请服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/applyInvoice.htm";
		// 开票申请服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/applyInvoice.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryInvoiceApplyRequest applyRequest = new GcbSalaryInvoiceApplyRequest();

		// 必填，订单号，必须唯一：建议14位以上时间戳+随机数
		applyRequest.setOrderId(request.getParameter("orderId"));

		// 必填，商户号，由我司提供的商户号+01，共13位数字
		applyRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，核心企业会员号，
		applyRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 必填，批次列表，代发批次号，多个英文逗号分隔，如1,2,3,4(多个表示合并开一张票)
		applyRequest.setBatchList(request.getParameter("batchList"));

		// 必填，异步地址，异步通知地址
		applyRequest.setAsynUrl(request.getParameter("asynUrl"));

		// 必填，接收人手机号，
		applyRequest.setReceiveMobile(request.getParameter("receiveMobile"));

		// 必填，接收人姓名，
		applyRequest.setReceiveName(request.getParameter("receiveName"));

		// 必填，快递地址，
		applyRequest.setCourierAddress(request.getParameter("courierAddress"));

		// 非必填，购方税号，
		applyRequest.setInvoiceTaxNo(request.getParameter("invoiceTaxNo"));

		// 非必填，购方公司名称，
		applyRequest.setInvoiceName(request.getParameter("invoiceName"));

		// 非必填，购方公司地址，
		applyRequest.setInvoiceAddress(request.getParameter("invoiceAddress"));

		// 非必填，购方公司账号，
		applyRequest.setInvoiceCardNum(request.getParameter("invoiceCardNum"));

		// 非必填，购方银行名称，
		applyRequest.setInvoiceBankName(request.getParameter("invoiceBankName"));

		// 非必填，购方联系电话，
		applyRequest.setInvoicePhone(request.getParameter("invoicePhone"));

		// 非必填，备注，
		applyRequest.setRemarks(request.getParameter("remarks"));

		// 发送请求
		GcbSalaryInvoiceApplyResponse applyResponse = gcbPayClient.execute(applyRequest);
		logger.info("GcbSalaryInvoiceApplyResponse => " + new String(applyResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", applyResponse.getParamMap());
		return "GcbSalaryInvoiceApplyResponse";
	}

	/**
	 * 步骤四、处理bg通知
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception
	 */
	@RequestMapping("/notifyReceiverBgInvoice.do")
	public String processBgNotification(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 实例化通知解析器
		GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

		// 解析通知结果
		GcbSalaryInvoiceResult result = gcbMessageParser.parse(request, GcbSalaryInvoiceResult.class);
		logger.info("GcbSalaryInvoiceResult =>" + result.getData());

		// 根据通知结果进行业务逻辑处理
		// ......

		// 向服务端返回: <result>1</result> ,表示结果接收成功，无需再通知。
		response.getWriter().write("<result>1</result>");

		return null;
	}

}
