package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.invoice.GcbSalaryInvoiceQueryRequest;
import com.huifu.jh.salary.model.invoice.GcbSalaryInvoiceQueryResponse;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * 开票结果查询
 */
@Controller
public class GcbSalaryInvoiceQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造开票结果查询信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryInvoiceQuery.do")
	public String showGcbSalaryInvoiceQuery(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryInvoiceQuery();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryInvoiceQuery";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryInvoiceQuery() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，核心企业机构号
		map.put("coreMemberId", "148079084486732398");

		// 非必填，开票批次号，开票批次号，可为空查询全部
		map.put("invoiceBatch", "");

		// 必填，订单号，原请求订单号
		map.put("orderId", "20211021194132408");

		return map;
	}
	

	/**
	 * 步骤二、构建开票结果查询信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryInvoiceQuery.do")
	public String doGcbSalaryInvoiceQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 开票结果查询服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/queryInvoiceInfo.htm";
		// 开票结果查询服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/queryInvoiceInfo.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryInvoiceQueryRequest queryRequest = new GcbSalaryInvoiceQueryRequest();

		// 必填，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，核心企业机构号
		queryRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 非必填，开票批次号，开票批次号，可为空查询全部
		queryRequest.setInvoiceBatch(request.getParameter("invoiceBatch"));

		// 必填，订单号，原请求订单号
		queryRequest.setOrderId(request.getParameter("orderId"));
		
		// 发送请求
		GcbSalaryInvoiceQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbSalaryInvoiceQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "GcbSalaryInvoiceQueryResponse";
	}

}
