package demo.salary.controller;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.credit.GcbSalaryPaymentApplyDetail;
import com.huifu.jh.salary.model.credit.GcbSalaryPaymentApplyRequest;
import com.huifu.jh.salary.model.credit.GcbSalaryPaymentApplyResponse;
import com.huifu.jh.salary.model.credit.GcbSalaryPaymentResult;
import demo.salary.config.AppConfig;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 工资代发
 */
@Controller
public class GcbSalaryPaymentApplyController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造工资代发信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryPaymentApply.do")
	public String showGcbSalaryPaymentApply(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryPaymentApply();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryPaymentApply";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryPaymentApply() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，请求流水号，
		map.put("orderId", new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));

		// 必填，核心企业会员号，
		map.put("coreMemberId", "148079084486732398");

		// 必填，落地公司会员号，
		map.put("groundMemberId", "980149419163571612");

		// 必填，代发模式，1-普票 2-专票（由核心企业和落地公司签的合同决定的）
		map.put("salaryModle", "1");

		// 必填，走账模式，1-直发 2-代发
		map.put("accountModle", "2");

		// 必填，走账类型，1-单账户模式；
		map.put("modeIdcompany", "1");

		// 必填，异步通知地址，
		map.put("noticeUrl", "http://localhost:8080/notifyReceiverBgPayment.do");

		// 必填，代发工资明细，明细参数参照下方参数字段
		// salaryMonth    月份yyyyMMdd    必填
		// name           姓名            必填
		// accountIdNo    身份证号        必填
		// contactTelNo   手机号          必填   手机号，限制大陆手机号，不支持境外
		// salary         工资金额        必填   单位为分，无小数位
		GcbSalaryPaymentApplyDetail detail = new GcbSalaryPaymentApplyDetail();
		detail.setSalaryMonth("202006");
		detail.setName("冯浩波");
		detail.setAccountIdNo("321322199301097215");
		detail.setContactTelNo("13218072315");
		detail.setSalary("500");

		GcbSalaryPaymentApplyDetail detail2 = new GcbSalaryPaymentApplyDetail();
		detail2.setSalaryMonth("202007");
		detail2.setName("冯浩波");
		detail2.setAccountIdNo("321322199301097215");
		detail2.setContactTelNo("13218072315");
		detail2.setSalary("500");

		List<GcbSalaryPaymentApplyDetail> batchSalaryModleDtos = new ArrayList<GcbSalaryPaymentApplyDetail>();
		batchSalaryModleDtos.add(detail);
		batchSalaryModleDtos.add(detail2);

		// 必填，代发工资明细，明细参数参照下方参数字段
		// salaryMonth    月份yyyyMMdd    必填
		// name           姓名            必填
		// accountIdNo    身份证号        必填
		// contactTelNo   手机号          必填   手机号，限制大陆手机号，不支持境外
		// salary         工资金额        必填   单位为分，无小数位
		map.put("batchSalaryModleDtos", JSON.toJSONString(batchSalaryModleDtos));

		return map;
	}
	

	/**
	 * 步骤二、构建工资代发信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryPaymentApply.do")
	public String doGcbSalaryPaymentApply(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 工资代发申请服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/salaryModle.htm";
		// 工资代发申请服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/salaryModle.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryPaymentApplyRequest applyRequest = new GcbSalaryPaymentApplyRequest();

		// 必填，商户号，汇付跨境系统商户号+01，共13位
		applyRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，请求流水号，
		applyRequest.setOrderId(request.getParameter("orderId"));

		// 必填，核心企业会员号，
		applyRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 必填，落地公司会员号，
		applyRequest.setGroundMemberId(request.getParameter("groundMemberId"));

		// 必填，代发模式，1-普票 2-专票（由核心企业和落地公司签的合同决定的）
		applyRequest.setSalaryModle(request.getParameter("salaryModle"));

		// 必填，走账模式，1-直发 2-代发
		applyRequest.setAccountModle(request.getParameter("accountModle"));

		// 必填，走账类型，1-单账户模式；
		applyRequest.setModeIdcompany(request.getParameter("modeIdcompany"));

		// 必填，异步通知地址，
		applyRequest.setNoticeUrl(request.getParameter("noticeUrl"));

		// 必填，代发工资明细，明细参数参照下方参数字段
		// salaryMonth    月份yyyyMMdd    必填
		// name           姓名            必填
		// accountIdNo    身份证号        必填
		// contactTelNo   手机号          必填   手机号，限制大陆手机号，不支持境外
		// salary         工资金额        必填   单位为分，无小数位
		applyRequest.setBatchSalaryModleDtos(this.buildBatchSalaryModleDtos(request.getParameter("batchSalaryModleDtos")));
		
		// 发送请求
		GcbSalaryPaymentApplyResponse applyResponse = gcbPayClient.execute(applyRequest);
		logger.info("GcbSalaryPaymentApplyResponse => " + new String(applyResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", applyResponse.getParamMap());
		return "GcbSalaryPaymentApplyResponse";
	}

	private List<GcbSalaryPaymentApplyDetail> buildBatchSalaryModleDtos(String batchSalaryModleDtosStr) {
		if (StringUtils.isBlank(batchSalaryModleDtosStr)) {
			return null;
		}

		List<GcbSalaryPaymentApplyDetail> batchSalaryModleDtos = JSON.parseArray(batchSalaryModleDtosStr, GcbSalaryPaymentApplyDetail.class);

		return batchSalaryModleDtos;

	}

	/**
	 * 步骤四、处理bg通知
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception
	 */
	@RequestMapping("/notifyReceiverBgPayment.do")
	public String processBgNotification(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 实例化通知解析器
		GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

		// 解析通知结果
		GcbSalaryPaymentResult result = gcbMessageParser.parse(request, GcbSalaryPaymentResult.class);
		logger.info("GcbSalaryPaymentResult =>" + result.getData());

		// 根据通知结果进行业务逻辑处理
		// ......

		// 向服务端返回: <result>1</result> ,表示结果接收成功，无需再通知。
		response.getWriter().write("<result>1</result>");

		return null;
	}

}
