package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.credit.GcbSalaryPaymentQueryRequest;
import com.huifu.jh.salary.model.credit.GcbSalaryPaymentQueryResponse;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * 工资代发结果查询
 */
@Controller
public class GcbSalaryPaymentQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造工资代发结果查询信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryPaymentQuery.do")
	public String showGcbSalaryPaymentQuery(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryPaymentQuery();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryPaymentQuery";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryPaymentQuery() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，核心企业会员号，核心企业会员号
		map.put("coreMemberId", "148079084486732398");

		// 条件必填，批次号，若订单号未填，则批次号必填
		map.put("attachId", "");

		// 条件必填，订单号，若批次号未填，则订单号必填
		map.put("orderId", "20211028103026629");

		return map;
	}
	

	/**
	 * 步骤二、构建工资代发结果查询信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryPaymentQuery.do")
	public String doGcbSalaryPaymentQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 工资代发结果查询服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/salaryResult.htm";
		// 工资代发结果查询服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/salaryResult.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryPaymentQueryRequest queryRequest = new GcbSalaryPaymentQueryRequest();

		// 必填，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，核心企业会员号，核心企业会员号
		queryRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 条件必填，批次号，若订单号未填，则批次号必填
		queryRequest.setAttachId(request.getParameter("attachId"));

		/// 条件必填，订单号，若批次号未填，则订单号必填
		queryRequest.setOrderId(request.getParameter("orderId"));
		
		// 发送请求
		GcbSalaryPaymentQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbSalaryPaymentQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "GcbSalaryPaymentQueryResponse";
	}

}
