package demo.salary.controller;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.member.*;
import demo.salary.config.AppConfig;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 员工信息修改
 */
@Controller
public class GcbSalaryStaffModifyController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造员工信息修改信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryStaffModify.do")
	public String showGcbSalaryStaffModify(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryStaffModify();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryStaffModify";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryStaffModify() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，订单号，必须唯一：建议14位以上时间戳+随机数
		map.put("orderId", new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));

		// 必填，核心企业会员号，核心企业机构号
		map.put("coreMemberId", "148079084486732398");

		// 必填，明细，
		// memberId        员工会员号   必填
		// contactTelNo    手机号       选填     手机号，限制大陆手机号，不支持境外。手机号/卡号至少一个有值
		// cardNum         卡号         选填     手机号/卡号至少一个有值
		GcbSalaryStaffModifyDetail detail = new GcbSalaryStaffModifyDetail();
		detail.setMemberId("555077972985925781");
		detail.setContactTelNo("18812341234");
		detail.setCardNum("111");

		GcbSalaryStaffModifyDetail detail2 = new GcbSalaryStaffModifyDetail();
		detail2.setMemberId("738874863665151524");
		detail2.setContactTelNo("18812341234");
		detail2.setCardNum("222");

		List<GcbSalaryStaffModifyDetail> batchModifyList = new ArrayList<GcbSalaryStaffModifyDetail>();
		batchModifyList.add(detail);
		batchModifyList.add(detail2);

		// 必填，明细，
		// memberId        员工会员号   必填
		// contactTelNo    手机号       选填     手机号，限制大陆手机号，不支持境外。手机号/卡号至少一个有值
		// cardNum         卡号         选填     手机号/卡号至少一个有值
		map.put("batchModifyList", JSON.toJSONString(batchModifyList));

		return map;
	}
	

	/**
	 * 步骤二、构建员工信息修改信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryStaffModify.do")
	public String doGcbSalaryStaffModify(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 员工信息修改服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/batchModifyAccount.htm";
		// 员工信息修改服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/batchModifyAccount.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryStaffModifyRequest modifyRequest = new GcbSalaryStaffModifyRequest();

		// 必填，商户号，汇付跨境系统商户号+01，共13位
		modifyRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，订单号，必须唯一：建议14位以上时间戳+随机数
		modifyRequest.setOrderId(request.getParameter("orderId"));

		// 必填，核心企业会员号，核心企业机构号
		modifyRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 必填，明细，
		// memberId        员工会员号   必填
		// contactTelNo    手机号       选填     手机号，限制大陆手机号，不支持境外。手机号/卡号至少一个有值
		// cardNum         卡号         选填     手机号/卡号至少一个有值
		modifyRequest.setBatchModifyList(this.buildBatchModifyList(request.getParameter("batchModifyList")));

		// 发送请求
		GcbSalaryStaffModifyResponse modifyResponse = gcbPayClient.execute(modifyRequest);
		logger.info("GcbSalaryStaffModifyResponse => " + new String(modifyResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", modifyResponse.getParamMap());
		return "GcbSalaryStaffModifyResponse";
	}

	private List<GcbSalaryStaffModifyDetail> buildBatchModifyList(String batchModifyListStr) {

		if (StringUtils.isBlank(batchModifyListStr)) {
			return null;
		}

		List<GcbSalaryStaffModifyDetail> batchModifyList = JSON.parseArray(batchModifyListStr, GcbSalaryStaffModifyDetail.class);

		return batchModifyList;
	}

}
