package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.member.GcbSalaryStaffQueryRequest;
import com.huifu.jh.salary.model.member.GcbSalaryStaffQueryResponse;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * 员工入驻查询
 */
@Controller
public class GcbSalaryStaffQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造员工入驻查询信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryStaffQuery.do")
	public String showGcbSalaryStaffQuery(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryStaffQuery();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryStaffQuery";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryStaffQuery() {

		Map<String, String> map = new HashMap<String, String>();

		// 条件必填，订单号，原请求流水号（当前不支持，请使用会员号查询）
		map.put("orderId", "");

		// 条件必填，会员号，请求流水号和会员号二选一
		map.put("personMemberId", "555077972985925781");

		return map;
	}
	

	/**
	 * 步骤二、构建员工入驻查询信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryStaffQuery.do")
	public String doGcbSalaryStaffQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 员工入驻查询服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/queryPersonnel.htm";
		// 员工入驻查询服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/queryPersonnel.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryStaffQueryRequest queryRequest = new GcbSalaryStaffQueryRequest();

		// 必填，商户号，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 条件必填，订单号，原请求流水号（当前不支持，请使用会员号查询）
		queryRequest.setOrderId(request.getParameter("orderId"));

		// 条件必填，会员号，请求流水号和会员号二选一
		queryRequest.setPersonMemberId(request.getParameter("personMemberId"));

		// 发送请求
		GcbSalaryStaffQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbSalaryStaffQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "GcbSalaryStaffQueryResponse";
	}

}
