package demo.salary.controller;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.member.GcbSalarySignUrlNotification;
import com.huifu.jh.salary.model.member.GcbSalaryStaffRegisterDetail;
import com.huifu.jh.salary.model.member.GcbSalaryStaffRegisterRequest;
import com.huifu.jh.salary.model.member.GcbSalaryStaffRegisterResponse;
import com.huifu.jh.salary.model.member.GcbSalarySignResult;
import demo.salary.config.AppConfig;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 员工批量开户
 */
@Controller
public class GcbSalaryStaffRegisterController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造员工批量开户信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryStaffRegister.do")
	public String showGcbSalaryStaffRegister(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryStaffRegister();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryStaffRegister";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryStaffRegister() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，批次请求流水号，
		map.put("orderId", new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));

		// 必填，核心企业会员号，
		map.put("coreMemberId", "148079084486732398");

		// 必填，落地公司机构号，
		map.put("groundAgentId", "L20200527173554262");

		// 必填，签约链接异步通知地址，
		map.put("noticeUrlContactLink", "http://locaohost:8080/notifyReceiverBgStaff.do");

		// 必填，签约结果异步通知地址，
		map.put("noticeUrlContactResult", "http://locaohost:8080/notifyReceiverBgStaff2.do");

		// 非必填，用户签约pg通知地址，
		map.put("appSignSuccessUrl", "http://locaohost:8080/notifyReceiverPg.do");

		// 必填，明细列表，json格式
		// name           姓名            必填
		// bankAcctName   卡号姓名        必填
		// contactTelNo   手机号          必填   手机号，限制大陆手机号，不支持境外
		// accountIdNo    持卡人证件号    必填
		// idType         证件类型        必填   1-身份证；2-港澳通行证；
		// cardNum        卡号            必填
		// bankId         银行ID          必填   银联颁布的银行代码，参考附录
		// reqTransId     请求流水号       必填
		GcbSalaryStaffRegisterDetail detail = new GcbSalaryStaffRegisterDetail();
		detail.setName("zhu");
		detail.setBankAcctName("zhu");
		detail.setContactTelNo("17366372672");
		detail.setAccountIdNo("320001199002021211");
		detail.setIdType("1");
		detail.setCardNum("143673");
		detail.setBankId("01020000");
		detail.setReqTransId("14331");

		GcbSalaryStaffRegisterDetail detail2 = new GcbSalaryStaffRegisterDetail();
		detail2.setName("zhu2");
		detail2.setBankAcctName("zhu2");
		detail2.setContactTelNo("17366372673");
		detail2.setAccountIdNo("320001199002021212");
		detail2.setIdType("1");
		detail2.setCardNum("143673");
		detail2.setBankId("01020000");
		detail2.setReqTransId("14332");

		List<GcbSalaryStaffRegisterDetail> batchOpenList = new ArrayList<GcbSalaryStaffRegisterDetail>();
		batchOpenList.add(detail);
		batchOpenList.add(detail2);

		// 必填，明细列表，json格式
		// name           姓名            必填
		// bankAcctName   卡号姓名        必填
		// contactTelNo   手机号          必填   手机号，限制大陆手机号，不支持境外
		// accountIdNo    持卡人证件号    必填
		// idType         证件类型        必填   1-身份证；2-港澳通行证；
		// cardNum        卡号            必填
		// bankId         银行ID          必填   银联颁布的银行代码，参考附录
		// reqTransId     请求流水号       必填
		map.put("batchOpenList", JSON.toJSONString(batchOpenList));

		return map;
	}
	

	/**
	 * 步骤二、构建员工批量开户信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryStaffRegister.do")
	public String doGcbSalaryStaffRegister(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 员工批量开户服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/batchOpenAccount.htm";
		// 员工批量开户服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/batchOpenAccount.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryStaffRegisterRequest registerRequest = new GcbSalaryStaffRegisterRequest();

		// 必填，商户号，汇付跨境系统商户号+01，共13位
		registerRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，批次请求流水号，
		registerRequest.setOrderId(request.getParameter("orderId"));

		// 必填，核心企业会员号，
		registerRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 必填，落地公司机构号，
		registerRequest.setGroundAgentId(request.getParameter("groundAgentId"));

		// 必填，签约链接异步通知地址，
		registerRequest.setNoticeUrlContactLink(request.getParameter("noticeUrlContactLink"));

		// 必填，签约结果异步通知地址，
		registerRequest.setNoticeUrlContactResult(request.getParameter("noticeUrlContactResult"));

		// 非必填，用户签约pg通知地址，
		registerRequest.setAppSignSuccessUrl(request.getParameter("appSignSuccessUrl"));

		// 必填，明细列表，json格式
		// name           姓名            必填
		// bankAcctName   卡号姓名        必填
		// contactTelNo   手机号          必填   手机号，限制大陆手机号，不支持境外
		// accountIdNo    持卡人证件号    必填
		// idType         证件类型        必填   1-身份证；2-港澳通行证；
		// cardNum        卡号            必填
		// bankId         银行ID          必填   银联颁布的银行代码，参考附录
		// reqTransId     请求流水号       必填
		registerRequest.setBatchOpenList(this.buildBatchOpenList(request.getParameter("batchOpenList")));

		// 发送请求
		GcbSalaryStaffRegisterResponse registerResponse = gcbPayClient.execute(registerRequest);
		logger.info("GcbSalaryStaffRegisterResponse => " + new String(registerResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", registerResponse.getParamMap());
		return "GcbSalaryStaffRegisterResponse";
	}

	private List<GcbSalaryStaffRegisterDetail> buildBatchOpenList(String batchOpenListStr) {
		if (StringUtils.isBlank(batchOpenListStr)) {
			return null;
		}

		List<GcbSalaryStaffRegisterDetail> batchOpenList = JSON.parseArray(batchOpenListStr, GcbSalaryStaffRegisterDetail.class);

		return batchOpenList;
	}

	/**
	 * 步骤四、处理bg通知
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception
	 */
	@RequestMapping("/notifyReceiverBgStaff.do")
	public String processBgNotification(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 实例化通知解析器
		GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

		// 解析通知结果
		GcbSalarySignUrlNotification result = gcbMessageParser.parse(request, GcbSalarySignUrlNotification.class);
		logger.info("GcbSalarySignUrlNotification =>" + result.getData());

		// 根据通知结果进行业务逻辑处理
		// ......

		// 向服务端返回: <result>1</result> ,表示结果接收成功，无需再通知。
		response.getWriter().write("<result>1</result>");

		return null;
	}

	/**
	 * 步骤四、处理bg通知
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception
	 */
	@RequestMapping("/notifyReceiverBgStaff2.do")
	public String processBgNotification2(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 实例化通知解析器
		GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

		// 解析通知结果
		GcbSalarySignResult result = gcbMessageParser.parse(request, GcbSalarySignResult.class);
		logger.info("GcbSalarySignResult =>" + result.getData());

		// 根据通知结果进行业务逻辑处理
		// ......

		// 向服务端返回: <result>1</result> ,表示结果接收成功，无需再通知。
		response.getWriter().write("<result>1</result>");

		return null;
	}

}
