package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.withdraw.GcbSalaryWithdrawApplyRequest;
import com.huifu.jh.salary.model.withdraw.GcbSalaryWithdrawApplyResponse;
import com.huifu.jh.salary.model.withdraw.GcbSalaryWithdrawResult;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 个人提现申请
 */
@Controller
public class GcbSalaryWithdrawApplyController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造个人提现申请信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryWithdrawApply.do")
	public String showGcbSalaryWithdrawApply(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryWithdrawApply();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryWithdrawApply";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryWithdrawApply() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，核心企业会员号，
		map.put("coreMemberId", "148079084486732398");

		// 必填，员工会员号，
		map.put("personMemberId", "248580562032924868");

		// 必填，请求流水号，建议14位以上时间戳+随机数
		map.put("orderId", new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));

		// 必填，提现金额，单位为分
		map.put("orderAmt", "200");

		// 必填，异步通知地址，提现异步通知，返回地址
		map.put("retUrl", "http://localhost:8080/notifyReceiverBgWithdraw.do");

		return map;
	}
	

	/**
	 * 步骤二、构建个人提现申请信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryWithdrawApply.do")
	public String doGcbSalaryWithdrawApply(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 个人提现申请服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/withdrawPerson.htm";
		// 个人提现申请服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/withdrawPerson.htm";

		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryWithdrawApplyRequest applyRequest = new GcbSalaryWithdrawApplyRequest();

		// 必填，商户号，汇付跨境系统商户号+01，共13位
		applyRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，核心企业会员号，
		applyRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 必填，员工会员号，
		applyRequest.setPersonMemberId(request.getParameter("personMemberId"));

		// 必填，请求流水号，建议14位以上时间戳+随机数
		applyRequest.setOrderId(request.getParameter("orderId"));

		// 必填，提现金额，单位为分
		applyRequest.setOrderAmt(request.getParameter("orderAmt"));

		// 必填，异步通知地址，提现异步通知，返回地址
		applyRequest.setRetUrl(request.getParameter("retUrl"));

		// 发送请求
		GcbSalaryWithdrawApplyResponse applyResponse = gcbPayClient.execute(applyRequest);
		logger.info("GcbSalaryWithdrawApplyResponse => " + new String(applyResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", applyResponse.getParamMap());
		return "GcbSalaryWithdrawApplyResponse";
	}

	/**
	 * 步骤四、处理bg通知
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception
	 */
	@RequestMapping("/notifyReceiverBgWithdraw.do")
	public String processBgNotification(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 实例化通知解析器
		GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

		// 解析通知结果
		GcbSalaryWithdrawResult result = gcbMessageParser.parse(request, GcbSalaryWithdrawResult.class);
		logger.info("GcbSalaryWithdrawResult =>" + result.getData());

		// 根据通知结果进行业务逻辑处理
		// ......

		// 向服务端返回: <result>1</result> ,表示结果接收成功，无需再通知。
		response.getWriter().write("<result>1</result>");

		return null;
	}

}
