package demo.salary.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.salary.model.withdraw.*;
import demo.salary.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * 个人提现结果查询
 */
@Controller
public class GcbSalaryWithdrawQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造个人提现结果查询信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showGcbSalaryWithdrawQuery.do")
	public String showGcbSalaryWithdrawQuery(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildGcbSalaryWithdrawQuery();
		
		model.addAllAttributes(orderInfoMap);
		
		return "GcbSalaryWithdrawQuery";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildGcbSalaryWithdrawQuery() {

		Map<String, String> map = new HashMap<String, String>();

		// 必填，核心企业会员号，
		map.put("coreMemberId", "148079084486732398");

		// 条件必填，订单号，若系统流水号未填，则订单号必填
		map.put("orderId", "245665634");

		// 条件必填，系统流水号，若订单号未填，则系统流水号必填
		map.put("transSeqId", "");

		return map;
	}
	

	/**
	 * 步骤二、构建个人提现结果查询信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doGcbSalaryWithdrawQuery.do")
	public String doGcbSalaryWithdrawQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

		// 个人提现结果查询服务的url(测试环境)
		String serverUrl = "https://hfgj.testpnr.com/salary/withdrawInquire.htm";
		// 个人提现结果查询服务的url(生产环境)
//	String serverUrl = "https://global.chinapnr.com/salary/withdrawInquire.htm";


		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

		// 构建请求对象
		GcbSalaryWithdrawQueryRequest queryRequest = new GcbSalaryWithdrawQueryRequest();

		// 必填，商户号，汇付跨境系统商户号+01，共13位
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId());

		// 必填，核心企业会员号，
		queryRequest.setCoreMemberId(request.getParameter("coreMemberId"));

		// 条件必填，订单号，若系统流水号未填，则订单号必填
		queryRequest.setOrderId(request.getParameter("orderId"));

		// 条件必填，系统流水号，若订单号未填，则系统流水号必填
		queryRequest.setTransSeqId(request.getParameter("transSeqId"));

		// 发送请求
		GcbSalaryWithdrawQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbSalaryWithdrawQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "GcbSalaryWithdrawQueryResponse";
	}

}
